require 'discounter/version'
require 'discounter/discount'

# We expect the entity to have a price attribute.
module Discounter
  class << self
    
    attr_accessor :price
    
    # Entity is the entity in which the price to be set is on.
    def initialize(entity)
      @entity ||= entity
      @price  ||= entity.try(:price)
    end
    
    def initialize_discount 
      # ?
    end
    
    # User is the user that may have a discount for this entity.
    # Join is the entity in which there is a setting for yes this user has a 
    # discount associated to them.
    # We expect the join model to have a discount_tier column that is set to either. 
    # So we expect it to be set up this way:
    # - Model Join - Attributes: user_id, discount_tier (optional column = entity_id (for discounting on)).
    # nil if there is no discount, or the discount tier if there is indeed one.
    def evaluate_discount(user, join)
      Join.find_all_by_user_id(user.id).each(&method(:verify_discount_applicability))
    end
    
    private
    # This method is passed a row from the join table to verify the discount on.
    def verify_discount_applicability(row)
      if row.try(:discount).nil?
        # There is no discount set for this user in the system
        @price
      else
        # There is a discount tier applied.  Let's do the calculation
        # And return out the price for the Interface.
        @price = (@price.to_f * row.discount.to_f).to_i
      end
    end
     
  end # / class
end # / module


# Interface:
# 
# Entity in this case is an event that has a purchase price
# That we sometimes want admins to be able to discount.
#
# event = Event.find(params[:id])
# 
# Safety check, as we expect a price on the model
# IDEA: Mixin an options hash that has price true/false flag on it.
# if event.price
#   discounter = Discounter.new(event)
#   
#   One big use case for this gem is to proxy the price IF we have a discounted
#   price available to it.
#   @event.price = discounter.evaluate_discount(user, join)
# end
#
#
#